import datetime

import pytest

from jasmine.console_formatter import ConsoleFormatter
from jasmine.js_api_parser import Parser
from jasmine.result_list import ResultList


@pytest.fixture
def results():
    parser = Parser()
    return parser.parse([
        {u'status': u'passed'},
        {u'status': u'failed'},
        {u'status': u'passed'},
        {u'status': u'pending', u'fullName': u'Context is this test is pending'},
        {u'status': u'disabled'},
    ])


@pytest.fixture
def passing_results():
    parser = Parser()
    return parser.parse([
        {u'status': u'passed'},
        {u'status': u'passed'},
        {u'status': u'pending', u'fullName': u'Context is this test is pending'},
    ])


@pytest.fixture
def browser_logs():
    return [
        {u'timestamp': 0, 'level': 'INFO', 'message': 'hi'},
        {u'timestamp': 1000, 'level': 'WARNING', 'message': 'lo'},
        {u'timestamp': 2000, 'level': 'INFO', 'message': 'bye'},
    ]


def _create_console_formatter(spec_results=None, suite_results=None, browser_logs=None, seed=None, colors=False):
    spec_results = spec_results or ResultList([])
    suite_results = suite_results or ResultList([])
    browser_logs = browser_logs or []

    return ConsoleFormatter(
        spec_results=spec_results,
        suite_results=suite_results,
        browser_logs=browser_logs,
        seed=seed,
        colors=colors,
    )


def test_format_progress(results):
    formatter = _create_console_formatter(spec_results=results, colors=False)

    assert formatter.format_progress() == ".X.*"


def test_format_summary(results):
    formatter = _create_console_formatter(spec_results=results, colors=False)

    assert formatter.format_summary() == "4 specs, 1 failed, 1 pending"


def test_format_summary_with_seed(results):
    formatter = _create_console_formatter(spec_results=results, seed=1234, colors=False)

    assert formatter.format_summary() == "4 specs, 1 failed, 1 pending\nRandomized with seed 1234 (jasmine-ci --seed 1234)"


def test_format_suite_errors_empty():
    formatter = _create_console_formatter(suite_results=ResultList([]))
    assert formatter.format_suite_failure() == ""


def test_format_suite_errors():
    parser = Parser()
    suite_results = parser.parse([
        {
            u'status': u'failed',
            u'failedExpectations': [
                {"message": "ahhh", "stack": "stack1"},
                {"message": "oh no!", "stack": "stack2"}
            ]
        },
        {
            u'status': u'failed',
            u'failedExpectations': [
                {"message": "boom", "stack": "stack3"},
            ]
        },
        {
            u'status': u'failed',
            u'failedExpectations': [
                {"message": "nope"},
            ]
        }
    ])

    formatter = _create_console_formatter(suite_results=suite_results)
    assert formatter.format_suite_failure() == \
           "Suite Failures:\n" \
           + "  ahhh\n" \
           + "  stack1\n" \
           + "  oh no!\n" \
           + "  stack2\n" \
           + "  boom\n" \
           + "  stack3\n" \
           + "  nope\n"


def test_format_browser_logs(results, browser_logs):
    formatter = _create_console_formatter(spec_results=results, colors=False, browser_logs=browser_logs)

    dt1, dt2, dt3 = map(datetime.datetime.fromtimestamp, range(3))
    assert formatter.format_browser_logs() == (
        "Browser Session Logs:\n" +
        "  [{0} - INFO] hi\n".format(dt1) +
        "  [{0} - WARNING] lo\n".format(dt2) +
        "  [{0} - INFO] bye\n".format(dt3) +
        "\n"
    )


def test_format_browser_logs_with_no_failures(passing_results, browser_logs):
    formatter = _create_console_formatter(spec_results=passing_results, colors=False, browser_logs=browser_logs)

    assert formatter.format_browser_logs() == ""


def test_format_failures():
    parser = Parser()

    stack1 = u"""Error: Expected 'Batman' to equal 'PANTS'.
        at stack (http://localhost:8888/__jasmine__/jasmine.js:1110)
        at http://localhost:8888/__spec__/global_spec.js:3"""

    stack2 = u"""Error: Expected 'Batman' to equal 'Superman'.
        at stack (http://localhost:8888/__jasmine__/jasmine.js:1110)
        at http://localhost:8888/__spec__/global_spec.js:6"""

    stack3 = u"""Error: Expected 'Justice' to equal 'Served'.
        at stack (http://localhost:8888/__jasmine__/jasmine.js:1110)
        at http://localhost:8888/__spec__/global_spec.js:9"""

    results = parser.parse([
        {u'status': u'passed', u'fullName': u'Context is this test passes'},
        {u'status': u'failed', u'fullName': u'Context is this test fails',
         u'failedExpectations': [{u'stack': stack1, u'message': 'Message1'}]},
        {u'status': u'failed', u'fullName': u'Context is this test also fails',
         u'failedExpectations': [{u'stack': stack2, u'message': 'Message2'},
                                 {u'stack': stack3, u'message': 'Message3'}]},
    ])

    formatter = _create_console_formatter(spec_results=results, colors=False)

    assert formatter.format_spec_failures() == \
           "Context is this test fails\n" + \
           "  Message1\n" + \
           "  Error: Expected 'Batman' to equal 'PANTS'.\n" + \
           "        at http://localhost:8888/__spec__/global_spec.js:3\n" + \
           "Context is this test also fails\n" + \
           "  Message2\n" + \
           "  Error: Expected 'Batman' to equal 'Superman'.\n" + \
           "        at http://localhost:8888/__spec__/global_spec.js:6\n" + \
           "  Message3\n" + \
           "  Error: Expected 'Justice' to equal 'Served'.\n" + \
           "        at http://localhost:8888/__spec__/global_spec.js:9\n"


def test_clean_stack(results):
    formatter = _create_console_formatter(spec_results=results, colors=False)

    dirty_stack = u"""Error: Expected 'Batman' to equal 'PANTS'.
        at stack (http://localhost:8888/__jasmine__/jasmine.js:1110)
        at http://localhost:8888/__spec__/global_spec.js:3"""

    assert formatter.clean_stack(dirty_stack) == """Error: Expected 'Batman' to equal 'PANTS'.
        at http://localhost:8888/__spec__/global_spec.js:3"""


def test_pending_stack(results):
    formatter = _create_console_formatter(spec_results=results, colors=False)

    assert formatter.format_pending() == "Context is this test is pending\n"


def test_pending_with_message():
    parser = Parser()

    results = parser.parse(
        [
            {
                u'status': u'pending',
                u'fullName': u'pending',
                u'pendingReason': 'the reason'
            }
        ]
    )

    formatter = _create_console_formatter(spec_results=results, colors=False)
    assert formatter.format_pending() == "pending\n  Reason: the reason\n"

def test_deprecation_warning():
    parser = Parser()

    specs = parser.parse(
        [
            {
                u'status': u'passed',
                u'fullName': u'Speccy',
                u'deprecationWarnings': [
                    {
                        u'message': u'spec deprecated',
                        u'stack': None
                    }
                ]
            }
        ]
    )

    suites = parser.parse(
        [
            {
                u'status': u'passed',
                u'fullName': u'Sweet',
                u'deprecationWarnings': [
                    {
                        u'message': u'suite deprecated',
                        u'stack': None
                    }
                ]
            }
        ]
    )

    formatter = _create_console_formatter(spec_results=specs, suite_results=suites, colors=False)
    assert formatter.format_deprecations() == \
            "Deprecations:\n" + \
            "Speccy\n" + \
            "  spec deprecated\n" + \
            "  \n" + \
            "Sweet\n" + \
            "  suite deprecated\n" +\
            "  \n"




from jasmine.result import Result


def test_result():
    status = 'status'
    full_name = 'fullName'
    failed_expectations = 'failedExpectations'
    runnable_id = 'id'
    description = 'description'
    pending_reason = 'pendingReason'
    deprecation_warnings = 'deprecationWarnings'

    result = Result(
        status=status,
        full_name=full_name,
        failed_expectations=failed_expectations,
        runnable_id=runnable_id,
        description=description,
        pending_reason=pending_reason,
        deprecation_warnings=deprecation_warnings
    )

    assert result.status == status
    assert result.full_name == full_name
    assert result.failed_expectations == failed_expectations
    assert result.runnable_id == runnable_id
    assert result.description == description
    assert result.pending_reason == pending_reason
    assert result.deprecation_warnings == deprecation_warnings



import mimetypes
import os
import cherrypy
from jinja2 import Environment
import pkg_resources


class JasmineApp(object):

    def __init__(self, jasmine_config=None):
        self.jasmine_config = jasmine_config
        self.jinja_env = Environment()
        self.jasmine_file = JasmineFile()

    @cherrypy.expose
    def index(self, **kwargs):
        self.jasmine_config.reload()
        context = {
            'css_files': self.jasmine_config.stylesheet_urls(),
            'js_files': self.jasmine_config.script_urls()
        }

        template = self.jinja_env.from_string(pkg_resources.resource_string(
            'jasmine.templates',
            'runner.html'
        ).decode())

        return template.render(**context)

    def _cp_dispatch(self, vpath):
        if vpath[0] == '__jasmine__':
            vpath.pop(0)
            if len(vpath) == 1 and vpath[0] == 'jasmine_favicon.png':
                vpath.pop()
                cherrypy.request.params['path'] = 'jasmine_favicon'
            else:
                cherrypy.request.params['path'] = "/".join(vpath)
                vpath[:] = []
            return self.jasmine_file

        return vpath

    def run(self, host='127.0.0.1', port=8888, blocking=False):
        cherrypy.config.update({
            'server.socket_host': host,
            'server.socket_port': port
        })
        cherrypy.tree.mount(self, '/', {
                '/__src__':
                {
                    'tools.staticdir.on': True,
                    'tools.staticdir.dir': os.path.abspath(self.jasmine_config.src_dir())
                },
                '/__spec__':
                {
                    'tools.staticdir.on': True,
                    'tools.staticdir.dir': os.path.abspath(self.jasmine_config.spec_dir())
                }
            }
        )
        cherrypy.engine.signals.subscribe
        cherrypy.engine.start()
        if blocking:
            cherrypy.engine.block()

    def stop(self):
        cherrypy.engine.stop()


class JasmineFile(object):
    @cherrypy.expose
    def index(self, path):
        if path == 'jasmine_favicon':
            return cherrypy.lib.static.serve_fileobj(
                pkg_resources.resource_stream(
                    'jasmine_core.images',
                    'jasmine_favicon.png'
                ),
                content_type='image/png'
            )
        mime_type, _ = mimetypes.guess_type(path)
        return cherrypy.lib.static.serve_fileobj(
            pkg_resources.resource_stream('jasmine_core', path),
            content_type=mime_type
        )



import urllib


class JasmineUrlBuilder(object):

    def __init__(self, jasmine_config):
        self.jasmine_config = jasmine_config

    def build_url(self, port, seed=None):
        netloc = "localhost:{0}".format(port)
        query_string = self._build_query_params(seed=seed)
        jasmine_url = urllib.parse.urlunparse(('http', netloc, "", "", query_string, ""))
        return jasmine_url

    def _build_query_params(self, seed):
        query_params = {
            "throwFailures": self.jasmine_config.stop_spec_on_expectation_failure(),
            "failFast": self.jasmine_config.stop_on_spec_failure(),
            "random": self.jasmine_config.random(),
            "seed": seed
        }
        query_params = self._remove_empty_params(query_params)
        return urllib.parse.urlencode(query_params)

    def _remove_empty_params(self, query_params):
        return dict(((k, str(v).lower()) for k, v in query_params.items() if k == "random" or v))



from jasmine.result import Result


class ResultList(list):

    def add_result(self, result):
        self.append(Result(**result))

    def passed(self):
        return self._filter_status('passed')

    def failed(self):
        return self._filter_status('failed')

    def pending(self):
        return self._filter_status('pending')

    def enabled(self):
        return [result for result in self if result.status != 'disabled']

    def _filter_status(self, status):
        return [result for result in self if result.status == status]

    def __add__(self, other):
        return ResultList(list.__add__(self, other))



class FakeConfig(object):
    def __init__(
            self,
            src_dir=None,
            spec_dir=None,
            stylesheet_urls=None,
            script_urls=None,
            stop_spec_on_expectation_failure=False,
            stop_on_spec_failure=False,
            random=True
    ):
        self._src_dir = src_dir
        self._spec_dir = spec_dir
        self._stylesheet_urls = stylesheet_urls
        self._script_urls = script_urls
        self._stop_spec_on_expectation_failure = stop_spec_on_expectation_failure
        self._stop_on_spec_failure = stop_on_spec_failure
        self._random = random

        self.reload_call_count = 0

    def src_dir(self):
        return self._src_dir

    def spec_dir(self):
        return self._spec_dir

    def stylesheet_urls(self):
        return self._stylesheet_urls

    def script_urls(self):
        return self._script_urls

    def stop_spec_on_expectation_failure(self):
        return self._stop_spec_on_expectation_failure

    def stop_on_spec_failure(self):
        return self._stop_on_spec_failure

    def random(self):
        return self._random

    def reload(self):
        self.reload_call_count += 1



class Result():
    def __init__(
            self,
            status=None,
            full_name=None,
            failed_expectations=None,
            deprecation_warnings=None,
            runnable_id=None,
            description=None,
            pending_reason=None
    ):
        if failed_expectations is None:
            failed_expectations = {}

        self._status = status
        self._full_name = full_name
        self._failed_expectations = failed_expectations
        self._deprecation_warnings = deprecation_warnings
        self._runnable_id = runnable_id
        self._description = description
        self._pending_reason = pending_reason

    @property
    def status(self):
        return self._status

    @property
    def full_name(self):
        return self._full_name

    @property
    def failed_expectations(self):
        return self._failed_expectations

    @property
    def deprecation_warnings(self):
        return self._deprecation_warnings

    @property
    def runnable_id(self):
        return self._runnable_id

    @property
    def description(self):
        return self._description

    @property
    def pending_reason(self):
        return self._pending_reason



import os

import pytest
from jasmine_core import Core
import pkg_resources

notwin32 = pytest.mark.skipif("sys.platform == 'win32'")


@notwin32
def test_js_files():
    files = [
        'jasmine.js',
        'jasmine-html.js',
        'json2.js',
        'node_boot.js',
        'boot.js'
    ]

    assert Core.js_files() == files


def test_css_files():
    """ Should return a list of css files that are relative to Core.path() """
    assert ['jasmine.css'] == Core.css_files()


def test_favicon():
    assert os.path.isfile(pkg_resources.resource_filename('jasmine_core.images', 'jasmine_favicon.png'))


from mock import Mock
import pytest

from jasmine.result_list import ResultList


@pytest.fixture
def result_list():
    return ResultList([
        Mock(status='passed', id='pass'),
        Mock(status='failed', id='fail'),
        Mock(status='pending', id='pend'),
        Mock(status='disabled', id='disable'),
    ])


def test_result_list_passed(result_list):
    passed = result_list.passed()
    assert len(passed) == 1
    assert passed[0].id == 'pass'


def test_result_list_failed(result_list):
    failed = result_list.failed()
    assert len(failed) == 1
    assert failed[0].id == 'fail'


def test_result_list_pending(result_list):
    pending = result_list.pending()
    assert len(pending) == 1
    assert pending[0].id == 'pend'


def test_result_list_enabled(result_list):
    enabled = result_list.enabled()
    assert len(enabled) == 3
    assert enabled[0].id == 'pass'
    assert enabled[1].id == 'fail'
    assert enabled[2].id == 'pend'


def test_result_list_concatination(result_list):
    concatinated = result_list + result_list
    assert len(concatinated.passed()) == 2



import os

from yaml import load, Loader
from jasmine_core import Core

from jasmine.utils import iglob


class Config(object):
    def __init__(self, yaml_file, project_path=None):
        self.yaml_file = yaml_file
        self.project_path = project_path or os.getcwd()
        self._load()

    def script_urls(self):
        core_js_files = Core.js_files()
        if 'node_boot.js' in core_js_files:
            core_js_files.remove('node_boot.js')
        return [
                   "/__jasmine__/{0}".format(core_js) for core_js in core_js_files
               ] + [
                   self._prefix_src_underscored(src_file) for src_file in self.src_files()
               ] + [
                   "/__spec__/{0}".format(helper) for helper in self.helpers()
               ] + [
                   "/__spec__/{0}".format(spec_file) for spec_file in self.spec_files()
               ]

    def stylesheet_urls(self):
        return [
                   "/__jasmine__/{0}".format(core_css) for core_css in Core.css_files()
               ] + [
                   "/__src__/{0}".format(css_file) for css_file in self.stylesheets()
               ]

    def reload(self):
        self._load()

    def src_files(self):
        return self._glob_filelist('src_files', self.src_dir())

    def stylesheets(self):
        return self._glob_filelist('stylesheets', self.src_dir())

    def helpers(self):
        default_helpers = os.path.join(self.project_path, self.spec_dir(),
                                       'helpers/**/*.js')

        return self._glob_filelist('helpers', self.spec_dir(),
                                   default=[default_helpers])

    def spec_files(self):
        default_spec = os.path.join(self.project_path, self.spec_dir(),
                                    "**/*[sS]pec.js")

        return self._glob_filelist('spec_files', self.spec_dir(),
                                   default=[default_spec])

    def src_dir(self):
        return self._yaml.get("src_dir") or self.project_path

    def spec_dir(self):
        return self._yaml.get("spec_dir") or 'spec/javascripts'

    def stop_spec_on_expectation_failure(self):
        return self._yaml.get("stop_spec_on_expectation_failure") is True

    def stop_on_spec_failure(self):
        return self._yaml.get("stop_on_spec_failure") is True

    def random(self):
        return self._yaml.get("random", True) is not False

    def _prefix_src_underscored(self, path):
        return (
            path if path.startswith('http') else "/__src__/{0}".format(path)
        )

    def _uniq(self, items, idfun=None):
        # order preserving

        if idfun is None:
            def idfun(x):
                return x
        seen = {}
        result = []
        for item in items:
            marker = idfun(item)
            # in old Python versions:
            # if seen.has_key(marker)
            # but in new ones:
            if marker in seen:
                continue

            seen[marker] = 1
            result.append(item)
        return result

    def _glob_filelist(self, filelist, relative_to, default=None):
        default = default or []
        paths = self._yaml.get(filelist) or default

        paths = [self._make_absolute(path, relative_to) for path in paths]
        paths = [self._expland_globs(path) for path in paths]
        paths = sum(paths, [])
        relpaths = [self._make_relative(path, relative_to) for path in paths]

        # fix py26 relpath from root bug http://bugs.python.org/issue5117
        return [
            relpath[3:] if relpath.startswith("../") else relpath
            for relpath in relpaths
        ]

    def _make_absolute(self, path, relative_to):
        return (
            path if path.startswith('http')
            else os.path.abspath(
                os.path.join(self.project_path, relative_to, path)
            )
        )

    def _expland_globs(self, path):
        return (
            [path] if path.startswith('http')
            else self._glob_paths([path])
        )

    def _make_relative(self, path, relative_to):
        return (
            path if path.startswith('http')
            else os.path.relpath(path, relative_to)
        )

    def _glob_paths(self, paths):
        files = []

        for src_glob in paths:
            files.extend([os.path.abspath(x) for x in iglob(src_glob)])

        return list(self._uniq(files))

    def _extract_urls(self, filelist):
        local_files = [x for x in filelist if 'http' not in x]
        urls = [x for x in filelist if 'http' in x]

        return local_files, urls

    def _load(self):
        with open(self.yaml_file, 'rU') as f:
            self._yaml = load(f, Loader=Loader) or {}



from jasmine.js_api_parser import Parser


def test_parser_should_return_a_correct_results_list():
    parser = Parser()

    results = parser.parse([
        {
            u'status': u'failed',
            u'fullName': u'Globals refer to the most holy.',
            u'failedExpectations': [
                {
                    u'actual': u'Batman',
                    u'matcherName': u'toEqual',
                    u'passed': False,
                    u'expected': u'PANTS',
                    u'message': u"Expected 'Batman' to equal 'PANTS'.",
                    u'stack': u"stack\n    stack\n    stack"
                }
            ],

            u'passedExpectations': [
                {
                    u'matcherName': u'toBeTruthy',
                    u'expected': [],
                    u'actual': True,
                    u'message': u'Passed.',
                    u'stack': u'',
                    u'passed': True
                }
            ],
            u'deprecationWarnings': [
                {
                    u'message': u'old and busted',
                    u'stack': u'snacky stack'
                }
            ],
            u'id': 0,
            u'description': u'refer to the most holy',
            u'pendingReason': u'pending reason'
        }
    ])

    assert len(results) == 1
    assert results[0].status == 'failed'
    assert results[0].full_name == 'Globals refer to the most holy.'
    assert len(results[0].failed_expectations) == 1
    assert results[0].failed_expectations[0]['stack'] == "stack\n    stack\n    stack"
    assert results[0].failed_expectations[0]['message'] == "Expected 'Batman' to equal 'PANTS'."
    assert results[0].pending_reason == u'pending reason'
    assert len(results[0].deprecation_warnings) == 1
    assert results[0].deprecation_warnings[0]['message'] == "old and busted"
    assert results[0].deprecation_warnings[0]['stack'] == "snacky stack"


def test_parser_returns_an_empty_results_list_with_no_runnables():
    parser = Parser()
    results = parser.parse([])

    assert len(results) == 0


def test_parser_returns_all_failed_expectations():
    parser = Parser()

    results = parser.parse([
        {
            u'failedExpectations': [
                {u'actual': u'Expectation1'},
                {u'actual': u'Expectation2'},
                {u'actual': u'Expectation3'},
                {u'actual': u'Expectation4'},
            ],
        }
    ])

    assert len(results) == 1
    assert len(results[0].failed_expectations) == 4
    assert results[0].failed_expectations[0][u'actual'] == u'Expectation1'
    assert results[0].failed_expectations[1][u'actual'] == u'Expectation2'
    assert results[0].failed_expectations[2][u'actual'] == u'Expectation3'
    assert results[0].failed_expectations[3][u'actual'] == u'Expectation4'


import os
import tempfile

from mock import Mock, MagicMock
import pytest
from yaml import load
from contextlib import contextmanager

import builtins
from jasmine.entry_points import Command, mkdir_p


@contextmanager
def pushd(dest):
    src = os.getcwd()
    os.chdir(dest)
    try:
        yield
    finally:
        os.chdir(src)

@contextmanager
def in_temp_dir():
    with tempfile.TemporaryDirectory() as root:
        with pushd(root):
            yield


class FakeApp(object):
    def __init__(self, jasmine_config=None):
        FakeApp.app = self
        self.run = MagicMock(name='run')

    def run(self, host=None, port=None, blocking=False):
        pass

class FakeCI(object):
    def __init__(self, jasmine_config=None):
        FakeCI.runner = self
        self.run = MagicMock(name='run')

    def run(self, browser=None, show_logs=False, seed=None, app=None):
        pass


def write_config(path="spec/javascripts/support/jasmine.yml"):
    mkdir_p(os.path.dirname(path))
    with open(path, "w") as f:
        f.write("""
            src_dir: src
            spec_dir: spec
            """)


def test_ci_config_check():
    with in_temp_dir():
        FakeCI.runner = None
        cmd = Command(FakeApp, FakeCI)
        cmd.run(['ci'])
        assert FakeCI.runner == None


def test_continuous_integration__set_browser():
    with in_temp_dir():
        write_config()
        cmd = Command(FakeApp, FakeCI)
        cmd.run(['ci', '--browser', 'firefox'])

        FakeCI.runner.run.assert_called_once_with(browser='firefox', show_logs=False, seed=None, app=FakeApp.app)


def test_continuous_integration__browser_default(monkeypatch):
    with in_temp_dir():
        write_config()
        cmd = Command(FakeApp, FakeCI)
        cmd.run(['ci'])

        FakeCI.runner.run.assert_called_once_with(browser=None, show_logs=False, seed=None, app=FakeApp.app)


def test_continuous_integration__show_logs(monkeypatch):
    with in_temp_dir():
        write_config()
        cmd = Command(FakeApp, FakeCI)
        cmd.run(['ci', '--logs'])

        FakeCI.runner.run.assert_called_once_with(show_logs=True, browser=None, seed=None, app=FakeApp.app)


def test_continuous_integration__set_seed(monkeypatch):
    with in_temp_dir():
        write_config()
        cmd = Command(FakeApp, FakeCI)
        cmd.run(['ci', '--seed', '1234'])

        FakeCI.runner.run.assert_called_once_with(seed="1234", show_logs=False, browser=None, app=FakeApp.app)

def test_continuous_integration__custom_config__from_argv(monkeypatch):
    with in_temp_dir():
        write_config("custom/path/to/jasmine.yml")
        fake_standalone = Mock()
        cmd = Command(fake_standalone, FakeCI)
        cmd.run(['ci', '-c', 'custom/path/to/jasmine.yml'])

        fake_standalone.assert_called_once()
        standalone_construction_kwargs = fake_standalone.call_args[1]
        called_with_config = standalone_construction_kwargs['jasmine_config'].yaml_file
        assert called_with_config.endswith("custom/path/to/jasmine.yml")


def test_standalone__set_host(monkeypatch):
    with in_temp_dir():
        write_config()
        cmd = Command(FakeApp, FakeCI)
        cmd.run(['server', '-o', '127.0.0.2'])

        FakeApp.app.run.assert_called_once_with(host="127.0.0.2", port=8888, blocking=True)


def test_standalone__set_port(monkeypatch):
    with in_temp_dir():
        write_config()
        cmd = Command(FakeApp, FakeCI)
        cmd.run(['server', '-p', '1337'])

        FakeApp.app.run.assert_called_once_with(host="127.0.0.1", port=1337, blocking=True)


def test_standalone__port_default(monkeypatch):
    with in_temp_dir():
        write_config()
        cmd = Command(FakeApp, FakeCI)
        cmd.run(['server'])

        FakeApp.app.run.assert_called_once_with(host="127.0.0.1", port=8888, blocking=True)


def test_standalone__port_invalid(monkeypatch):
    with in_temp_dir():
        write_config()
        FakeApp.app = None
        cmd = Command(FakeApp, FakeCI)

        with pytest.raises(SystemExit):
            cmd.run(['server', '-p', 'pants'])

        assert "invalid int value: 'pants'"
        assert FakeApp.app == None

def test_standalone__missing_config(monkeypatch):
    with in_temp_dir():
        FakeApp.app = None
        cmd = Command(FakeApp, FakeCI)
        cmd.run(['server'])

        assert FakeApp.app == None

def test_standalone__custom_config__from_argv(monkeypatch):
    with in_temp_dir():
        write_config("custom/path/to/jasmine.yml")
        fake_standalone = Mock()
        cmd = Command(fake_standalone, FakeCI)
        cmd.run(['server', "-c", "custom/path/to/jasmine.yml"])

        fake_standalone.assert_called_once()
        standalone_construction_kwargs = fake_standalone.call_args[1]
        called_with_config = standalone_construction_kwargs['jasmine_config'].yaml_file
        assert called_with_config.endswith("custom/path/to/jasmine.yml")

def test_standalone__custom_config__when_environment_variable_set(monkeypatch):
    with in_temp_dir():
        write_config("custom/path/to/jasmine.yml")
        env_vars = {'JASMINE_CONFIG_PATH': "custom/path/to/jasmine.yml"}
        monkeypatch.setattr(os, 'environ', env_vars)
        fake_standalone = Mock()

        cmd = Command(fake_standalone, FakeCI)
        cmd.run(['server'])

        fake_standalone.assert_called_once()
        standalone_construction_kwargs = fake_standalone.call_args[1]
        called_with_config = standalone_construction_kwargs['jasmine_config'].yaml_file
        assert called_with_config.endswith("custom/path/to/jasmine.yml")


def test__query__yes(capsys, monkeypatch):
    cmd = Command(None, None)
    input_string(monkeypatch, "Y")

    choice = cmd.query("Would you like to play a game?")

    out, err = capsys.readouterr()

    assert out == "Would you like to play a game? [Y/n] "
    assert True == choice


def test__query__no(capsys, monkeypatch):
    cmd = Command(None, None)
    input_string(monkeypatch, "N")

    choice = cmd.query("Would you like to hear some cat facts?")

    out, err = capsys.readouterr()

    assert out == "Would you like to hear some cat facts? [Y/n] "
    assert False == choice


def test__query__default(capsys, monkeypatch):
    cmd = Command(None, None)
    input_string(monkeypatch, "")

    choice = cmd.query("Would you like to blindly accept my defaults?")

    out, err = capsys.readouterr()

    assert out == "Would you like to blindly accept my defaults? [Y/n] "
    assert True == choice


def test__mkdir_p():
    with in_temp_dir():
        mkdir_p("pants/a/b/c")
    
        assert os.path.isdir("pants")
        assert os.path.isdir("pants/a")
        assert os.path.isdir("pants/a/b")
        assert os.path.isdir("pants/a/b/c")


def input_string(monkeypatch, string=""):
    try:
        monkeypatch.setattr(builtins, 'raw_input', lambda: string)
    except AttributeError:
        monkeypatch.setattr(builtins, 'input', lambda: string)


def test_init__yes(monkeypatch):
    with in_temp_dir():
        # Should create spec/javascripts/support
        # Should create a spec/javascripts/support/jasmine.yml
        spec_dir = "spec/javascripts/support"
        yaml_file = os.path.join(spec_dir, "jasmine.yml")

        input_string(monkeypatch, "Y")

        Command(None, None).init(None)

        assert os.path.isdir(spec_dir)
        assert os.path.isfile(yaml_file)

        yaml = load(open(yaml_file))

        assert yaml['spec_files'] == ["**/*[Ss]pec.js"]


def test_init__yes__existing_yaml(monkeypatch):
    with in_temp_dir():
        mkdir_p("spec/javascripts/support")
        with open("spec/javascripts/support/jasmine.yml", "w") as f:
            f.write("""
                spec_files:
                    - "**/pants.*"
                """)
        # Should NOT overwrite spec/javascripts/support/jasmine.yml
        spec_dir = "spec/javascripts/support"
        yaml_file = os.path.join(spec_dir, "jasmine.yml")
    
        input_string(monkeypatch, "Y")
    
        Command(None, None).init(None)
    
        assert os.path.isdir(spec_dir)
        assert os.path.isfile(yaml_file)
    
        yaml = load(open(yaml_file))
    
        assert yaml['spec_files'] == ["**/pants.*"]


def test_init__no(monkeypatch):
    with in_temp_dir():
        # Should NOT create spec/javascripts/support
        # Should NOT create a spec/javascripts/support/jasmine.yml
        spec_dir = "spec/javascripts/support"
        yaml_file = os.path.join(spec_dir, "jasmine.yml")
    
        input_string(monkeypatch, "N")
    
        Command(None, None).init(None)
    
        assert not os.path.isdir(spec_dir)
        assert not os.path.isfile(yaml_file)


def test_init__run(monkeypatch):
    with in_temp_dir():
        cmd = Command(FakeApp, FakeCI)
        input_string(monkeypatch, "N")

        cmd.run(['init'])



import os
import tempfile
import pkg_resources
from contextlib import contextmanager

from jasmine.config import Config

default_jasmine_yml = """
    src_files:
      - src/player.js
      - src/**/*.js
      - http://cdn.jquery.com/jquery.js
      - vendor/test.js
      - vendor/**/*.{js,coffee}
    """

def create_files(jasmine_yml=default_jasmine_yml):
    files = {
        "jasmine.yml": jasmine_yml,
        "spec/javascripts/helpers/spec_helper.js": '',
        "lib/jams/jam_spec.js": '',
        "src/player.js": '',
        "src/mixer/mixer.js": '',
        "src/tuner/fm/fm_tuner.js": '',
        "spec/javascripts/player_spec.js": '',
        "spec/javascripts/mixer/mixer_spec.js": '',
        "spec/javascripts/tuner/fm/fm_tuner_spec.js": '',
        "spec/javascripts/tuner/am/AMSpec.js": '',
        "vendor/test.js": '',
        "vendor/pants.coffee": '',
        "vendor_spec/pantsSpec.js": '',
        "main.css": '',
    }
    for k in files:
        parent = os.path.dirname(k)
        if parent and not os.path.exists(parent):
            os.makedirs(parent)
        with open(k, 'w') as f:
            f.write(files[k])

@contextmanager
def pushd(dest):
    src = os.getcwd()
    os.chdir(dest)
    try:
        yield
    finally:
        os.chdir(src)

@contextmanager
def in_temp_dir():
    with tempfile.TemporaryDirectory() as root:
        with pushd(root):
            yield

def test_src_files():
    with in_temp_dir():
        create_files()
        config = Config("jasmine.yml")
        src_files = config.src_files()
    
        assert src_files[0] == "src/player.js"
        assert src_files.index("vendor/test.js") < src_files.index("vendor/pants.coffee")
    
        assert 'http://cdn.jquery.com/jquery.js' in src_files
        assert 'src/mixer/mixer.js' in src_files
        assert 'src/tuner/fm/fm_tuner.js' in src_files
        assert 'vendor/pants.coffee' in src_files


def test_stylesheets_default():
    with in_temp_dir():
        create_files()
        config = Config("jasmine.yml")
        assert config.stylesheets() == []


def test_helpers_default():
    with in_temp_dir():
        create_files()
        config = Config("jasmine.yml")
        assert config.helpers() == ['helpers/spec_helper.js']


def test_spec_files_default():
    with in_temp_dir():
        create_files()
        config = Config("jasmine.yml")
        # sort because all of the specified paths are globs, order does not matter
        assert sorted(config.spec_files()) == [
            'mixer/mixer_spec.js',
            'player_spec.js',
            'tuner/am/AMSpec.js',
            'tuner/fm/fm_tuner_spec.js',
        ]


def test_src_dir_spec_dir():
    with in_temp_dir():
        create_files("""
            src_dir: src
            spec_dir: spec
            src_files:
                - ./**/*.js
                - player.js
                - vendor/test.js
                - vendor/**/*.{js,coffee}
            """)
        config = Config("jasmine.yml")
        src_files = config.src_files()
    
        assert 'player.js' in src_files
        assert 'mixer/mixer.js' in src_files
        assert 'tuner/fm/fm_tuner.js' in src_files
    
        # noinspection PySetFunctionToLiteral
        assert set(config.spec_files()) == set([
            "javascripts/player_spec.js",
            "javascripts/mixer/mixer_spec.js",
            "javascripts/tuner/am/AMSpec.js",
            "javascripts/tuner/fm/fm_tuner_spec.js",
        ])


def test_script_urls(monkeypatch):
    monkeypatch.setattr(
        pkg_resources,
        'resource_listdir',
        lambda package, directory: [
            'json2.js',
            'jasmine.js',
            'boot.js',
            'node_boot.js',
            'jasmine-html.js',
            'jasmine.css'
        ]
    )

    with in_temp_dir():
        create_files()
        config = Config("jasmine.yml")
        script_urls = config.script_urls()

        assert script_urls[:5] == [
            "/__jasmine__/jasmine.js",
            "/__jasmine__/jasmine-html.js",
            "/__jasmine__/json2.js",
            "/__jasmine__/boot.js",
            "/__src__/src/player.js"
        ]
    
        assert 'http://cdn.jquery.com/jquery.js' in script_urls
        assert '/__src__/src/mixer/mixer.js' in script_urls
        assert '/__src__/src/tuner/fm/fm_tuner.js' in script_urls
        assert '/__src__/vendor/pants.coffee' in script_urls


def test_stylesheet_urls():
    with in_temp_dir():
        create_files("""
            stylesheets:
              - ./**/*.css
            """)
        with open("main.css", "r"):
            pass
        config = Config("jasmine.yml")

        stylesheet_urls = config.stylesheet_urls()
    
        assert stylesheet_urls == [
            "/__jasmine__/jasmine.css",
            "/__src__/main.css"
        ]

def test_stop_spec_on_expectation_failure_default():
    with in_temp_dir():
        create_files()
        config = Config("jasmine.yml")
        assert config.stop_spec_on_expectation_failure() is False


def test_stop_spec_on_expectation_failure_invalid():
    with in_temp_dir():
        create_files("""
                stop_spec_on_expectation_failure: pants
            """)
        config = Config("jasmine.yml")
        assert config.stop_spec_on_expectation_failure() is False


def test_stop_spec_on_expectation_failure_set():
    with in_temp_dir():
        create_files("""
                stop_spec_on_expectation_failure: true
            """)
        config = Config("jasmine.yml")
        assert config.stop_spec_on_expectation_failure() is True


def test_stop_on_spec_failure_default():
    with in_temp_dir():
        create_files()
        config = Config("jasmine.yml")
        assert config.stop_on_spec_failure() is False


def test_stop_on_spec_failure_invalid():
    with in_temp_dir():
        create_files("""
                stop_on_spec_failure: pants
            """)
        config = Config("jasmine.yml")
        assert config.stop_on_spec_failure() is False


def test_stop_on_spec_failure_set():
    with in_temp_dir():
        create_files("""
                stop_on_spec_failure: true
            """)
        config = Config("jasmine.yml")
        assert config.stop_on_spec_failure() is True

def test_random_default():
    with in_temp_dir():
        create_files()
        config = Config("jasmine.yml")
    assert config.random() is True


def test_random_invalid():
    with in_temp_dir():
        create_files("""
            random: pants
            """)
        config = Config("jasmine.yml")
        assert config.random() is True


def test_random_set_false():
    with in_temp_dir():
        create_files("""
            random: false
            """)
        config = Config("jasmine.yml")
        assert config.random() is False

def test_reload():
    with in_temp_dir():
        create_files()
        config = Config("jasmine.yml")
        assert config.src_files() != ['pants.txt']

        with open("jasmine.yml", "w") as f:
            f.write("""
                src_files:
                  - pants.txt
                """)

        with open("pants.txt", "w"):
            pass

        config.reload()
        assert config.src_files() == ['pants.txt']



import datetime


class ConsoleFormatter(object):
    COLORS = {
        'red': "\033[0;31m",
        'green': "\033[0;32m",
        'yellow': "\033[0;33m",
        'none': "\033[0m"
    }

    JASMINE_HEADER = """
      _                     _
     | |                   (_)
     | | __ _ ___ _ __ ___  _ _ __   ___
 _   | |/ _` / __| '_ ` _ \| | '_ \ / _ \\
| |__| | (_| \__ \ | | | | | | | | |  __/
 \____/ \__,_|___/_| |_| |_|_|_| |_|\___|

"""

    def __init__(self, spec_results, suite_results, browser_logs, seed=None, colors=True):
        self.results = spec_results
        self.suite_results = suite_results
        self.browser_logs = browser_logs
        self.colors = colors
        self.seed = seed

    def format(self):
        return (
            self.JASMINE_HEADER +
            self.format_progress() + "\n\n" +
            self.format_summary() + "\n\n" +
            self.format_suite_failure() +
            self.format_browser_logs() +
            self.format_spec_failures() +
            self.format_pending() +
            self.format_deprecations()
        )

    def format_progress(self):
        output = ""

        for result in self.results:
            if result.status == "passed":
                output += self._colorize('green', '.')
            elif result.status == "failed":
                output += self._colorize('red', 'X')
            elif result.status == 'pending':
                output += self._colorize('yellow', '*')

        return output

    def format_summary(self):
        output = "{0} specs, {1} failed".format(
            self._enabled_spec_count(),
            len(self.results.failed())
        )

        if self.results.pending():
            output += ", {0} pending".format(len(self.results.pending()))

        if self.seed:
            output += "\nRandomized with seed {0} (jasmine-ci --seed {0})".format(self.seed)
        return output

    def _enabled_spec_count(self):
        return len(self.results.enabled())

    def format_suite_failure(self):
        output = ""
        for failed_suite in self.suite_results.failed():
            for expectation in failed_suite.failed_expectations:
                output += self._format_expectation_failure(expectation)
        if output:
            output = self._colorize('red', 'Suite Failures:\n') + output
        return output

    def format_spec_failures(self):
        output = ""
        for failed_spec in self.results.failed():
            output += self._colorize('red', failed_spec.full_name) + "\n"
            for expectation in failed_spec.failed_expectations:
                output += self._format_expectation_failure(expectation)
        return output

    def format_browser_logs(self):
        output = ""
        if self.results.failed() and self.browser_logs:
            output = "Browser Session Logs:\n"
            for log in self.browser_logs:
                output += "  [{0} - {1}] {2}\n".format(
                    datetime.datetime.fromtimestamp(log['timestamp'] / 1000.0),
                    log['level'],
                    log['message']
                )
            output += "\n"
        return output

    def clean_stack(self, stack):
        if not stack:
            return ""

        def _dirty(stack_line):
            return "__jasmine__" in stack_line or "__boot__" in stack_line

        return "\n".join([
            stack_line
            for stack_line in stack.split("\n")
            if not _dirty(stack_line)
        ])

    def format_pending(self):
        output = ""

        for pending in self.results.pending():
            output += self._colorize('yellow', pending.full_name + "\n")
            if pending.pending_reason:
                output += "  Reason: {0}\n".format(pending.pending_reason)

        return output

    def format_deprecations(self):
        output = ""
        spec_deprecations = [result for result in self.results if result.deprecation_warnings]
        for deprecation in spec_deprecations:
            output += self._colorize('yellow', deprecation.full_name + "\n")
            for warning in deprecation.deprecation_warnings:
                output += self._format_expectation_failure(warning)
        suite_deprecations = [result for result in self.suite_results if result.deprecation_warnings]
        for deprecation in suite_deprecations:
            output += self._colorize('yellow', deprecation.full_name + "\n")
            for warning in deprecation.deprecation_warnings:
                output += self._format_expectation_failure(warning)
        if output:
            output = 'Deprecations:\n' + output

        return output

    def _format_expectation_failure(self, expectation):
            output = "  " + expectation['message'] + "\n"
            if 'stack' in expectation:
                output += "  " + self.clean_stack(expectation['stack']) + "\n"
            return output

    def _colorize(self, color, text):
        if not self.colors:
            return text

        return self.COLORS[color] + text + self.COLORS['none']



import datetime
import sys
import time
import urllib

import pytest
from mock import MagicMock

from jasmine.ci import CIRunner
from test.helpers.fake_config import FakeConfig


@pytest.fixture
def sysexit(monkeypatch):
    mock_exit = MagicMock()
    monkeypatch.setattr(sys, 'exit', mock_exit)
    return mock_exit


@pytest.fixture
def test_server(monkeypatch):
    import jasmine.ci
    instance = [None]

    class FakeTestServerThread(object):
        def __init__(self, port, app=None, *args, **kwargs):
            self.port = port
            instance[0] = self
        def start(arg):
            pass
        def join(self):
            pass

    monkeypatch.setattr(jasmine.ci, 'TestServerThread', FakeTestServerThread)
    return lambda: instance[0]


@pytest.fixture
def firefox_driver(monkeypatch):
    import selenium.webdriver.firefox.webdriver

    driver = MagicMock()
    driver_class = lambda: driver
    monkeypatch.setattr(selenium.webdriver.firefox.webdriver, 'WebDriver', driver_class)
    return driver


@pytest.fixture
def suites():
    return [
        {
            "id": 0,
            "name": "datepicker",
            "type": "suite",
            "children": [
                {
                    "id": 0,
                    "name": "calls the datepicker constructor",
                    "type": "spec",
                    "children": []
                },
                {
                    "id": 1,
                    "name": "icon triggers the datepicker",
                    "type": "spec",
                    "children": []
                }
            ]
        }
    ]


@pytest.fixture
def results():
    return [
        {
            "id": "spec0",
            "description": "",
            "fullName": "",
            "status": "failed",
            'failedExpectations': [
                {
                    "matcherName": "toHaveBeenCalledWith",
                    "expected": {
                        "format": "yyy-mm-dd"
                    },
                    "actual": {},
                    "message": "Expected spy datepicker to have been called with [ { format : 'yyy-mm-dd' } ] but actual calls were [ { format : 'yyyy-mm-dd' } ]",
                    "stack": "Totally the one you want",
                    "passed": False
                }

            ]
        },
        {
            "id": "spec1",
            "description": "",
            "fullName": "",
            "status": "passed",
            "failedExpectations": []
        }
    ]


@pytest.fixture
def suite_results():
    return [
        {
            "id": "suite0",
            "status": "failed",
            "failedExpectations": [
                {
                    "message": "something went wrong",
                    "stack": "stack"
                }
            ]
        },
        {
            "id": "suite1",
            "status": "finished"
        }
    ]


@pytest.fixture
def run_details():
    return {
        "overallStatus": "passed",
        "failedExpectations": [],
        "order": {
            "random": False,
            "seed": 54321
        }
    }


@pytest.fixture
def run_details_with_failures():
    return {
        "overallResult": "failed",
        "failedExpectations": [
            {
                "matcherName": "toEqual",
                "message": "Expected 'foo' to equal 'bar'.",
                "stack": "stack trace",
                "passed": False,
                "expected": "foo",
                "actual": "bar"
            },
            {
                "matcherName": "toEqual",
                "message": "Expected 'baz' to equal 'quux'.",
                "stack": "stack trace 2",
                "passed": False,
                "expected": "baz",
                "actual": "quux"
            }
        ],
        "order": {
            "random": False,
            "seed": 54321
        }
    }

@pytest.fixture
def run_details_random():
    return {
        "failedExpectations": [],
        "order": {
            "random": True,
            "seed": 12345
        }
    }


@pytest.fixture
def jasmine_config():
    return FakeConfig(
        src_dir='src',
        spec_dir='spec',
    )

def test_run_builds_url(suites, results, run_details, capsys, sysexit, firefox_driver, test_server, jasmine_config):
    CIRunner(jasmine_config=jasmine_config).run()
    firefox_driver.get.assert_called_with('http://localhost:%s?random=true' % test_server().port)


def test_run_with_random_seed(suites, results, run_details, capsys, sysexit, firefox_driver, test_server, jasmine_config):
    CIRunner(jasmine_config=jasmine_config).run(seed="1234")
    firefox_driver.get.assert_called()
    url = firefox_driver.get.call_args[0][0]
    uri_tuple = urllib.parse.urlparse(str(url))
    params = urllib.parse.parse_qs(uri_tuple[4])
    assert params['seed'] == ['1234']
    assert params['random'] == ['true']


def test_run_exits_with_zero_on_success(suites, run_details, capsys, sysexit, firefox_driver, test_server, jasmine_config):
    def execute_script(js):
        if 'jsApiReporter.finished' in js:
            return True
        if 'jsApiReporter.specResults' in js:
            return []
        if 'jsApiReporter.suiteResults' in js:
            return []
        if 'jsApiReporter.runDetails' in js:
            return run_details
        return None

    def get_log(type):
        return [dict(timestamp=0, level='INFO', message='hello')]

    firefox_driver.execute_script = execute_script
    firefox_driver.get_log = get_log

    CIRunner(jasmine_config=jasmine_config).run(show_logs=True)
    capsys.readouterr()

    assert not sysexit.called
    stdout, _ = capsys.readouterr()

    dt = datetime.datetime.fromtimestamp(0)
    assert '[{0} - INFO] hello\n'.format(dt) not in stdout


def test_run_exits_with_nonzero_on_failure(suites, run_details, capsys, sysexit, firefox_driver, test_server, jasmine_config):
    run_details['overallStatus'] = 'something other than passed'
    def execute_script(js):
        if 'jsApiReporter.finished' in js:
            return True
        if 'jsApiReporter.specResults' in js:
            return []
        if 'jsApiReporter.suiteResults' in js:
            return []
        if 'jsApiReporter.runDetails' in js:
            return run_details
        return None

    timestamp = time.time() * 1000

    def get_log(type):
        assert type == 'browser'
        return [
            dict(timestamp=timestamp, level='INFO', message='hello'),
            dict(timestamp=timestamp + 1, level='WARNING', message='world'),
        ]

    firefox_driver.execute_script = execute_script
    firefox_driver.get_log = get_log

    CIRunner(jasmine_config=jasmine_config).run()

    sysexit.assert_called_with(1)
    stdout, _ = capsys.readouterr()

    assert "Browser Session Logs" not in stdout
    assert "hello" not in stdout
    assert "world" not in stdout


def test_run_with_browser_logs(suites, results, run_details, capsys, sysexit, firefox_driver, test_server, jasmine_config):
    def execute_script(js):
        if 'jsApiReporter.finished' in js:
            return True
        if 'jsApiReporter.specResults' in js:
            return results
        if 'jsApiReporter.suiteResults' in js:
            return []
        if 'jsApiReporter.runDetails' in js:
            return run_details
        return None

    timestamp = time.time() * 1000

    def get_log(type):
        assert type == 'browser'
        return [
            dict(timestamp=timestamp, level='INFO', message='hello'),
            dict(timestamp=timestamp + 1, level='WARNING', message='world'),
        ]

    firefox_driver.execute_script = execute_script
    firefox_driver.get_log = get_log

    CIRunner(jasmine_config=jasmine_config).run(show_logs=True)

    stdout, _ = capsys.readouterr()

    dt = datetime.datetime.fromtimestamp(timestamp / 1000.0)
    assert '[{0} - INFO] hello\n'.format(dt) in stdout

    dt = datetime.datetime.fromtimestamp((timestamp + 1) / 1000.0)
    assert '[{0} - WARNING] world\n'.format(dt) in stdout


def test_displays_afterall_errors(suite_results, suites, results, run_details, capsys, sysexit, firefox_driver, test_server, jasmine_config):
    results[0] = results[1]
    del results[1]

    def execute_script(js):
        if 'jsApiReporter.finished' in js:
            return True
        if 'jsApiReporter.specResults' in js:
            return results
        if 'jsApiReporter.suiteResults' in js:
            return suite_results
        if 'jsApiReporter.runDetails' in js:
            return run_details
        return None

    firefox_driver.execute_script = execute_script

    CIRunner(jasmine_config=jasmine_config).run()
    stdout, _ = capsys.readouterr()

    assert 'something went wrong' in stdout


def test_displays_incomplete_reason(suite_results, suites, run_details, capsys, sysexit, firefox_driver, test_server, jasmine_config):
    run_details['overallStatus'] = 'incomplete'
    run_details['incompleteReason'] = 'out of cheese'

    def execute_script(js):
        if 'jsApiReporter.finished' in js:
            return True
        if 'jsApiReporter.specResults' in js:
            return []
        if 'jsApiReporter.suiteResults' in js:
            return suite_results
        if 'jsApiReporter.runDetails' in js:
            return run_details
        return None

    firefox_driver.execute_script = execute_script

    CIRunner(jasmine_config=jasmine_config).run()
    stdout, _ = capsys.readouterr()

    assert 'Incomplete: out of cheese' in stdout


def test_displays_top_suite_errors(suite_results, suites, results, run_details_with_failures, capsys, sysexit, firefox_driver, test_server, jasmine_config):
    results[0] = results[1]
    del results[1]

    def execute_script(js):
        if 'jsApiReporter.finished' in js:
            return True
        if 'jsApiReporter.specResults' in js:
            return results
        if 'jsApiReporter.suiteResults' in js:
            return suite_results
        if 'jsApiReporter.runDetails' in js:
            return run_details_with_failures
        return None

    firefox_driver.execute_script = execute_script

    CIRunner(jasmine_config=jasmine_config).run()
    stdout, _ = capsys.readouterr()

    assert "Expected 'foo' to equal 'bar'." in stdout
    assert "stack trace" in stdout
    assert "Expected 'baz' to equal 'quux'." in stdout
    assert "stack trace 2" in stdout

    sysexit.assert_called_with(1)


def test_displays_random_seed(results, suite_results, run_details_random, firefox_driver, capsys, test_server, jasmine_config, sysexit):
    def execute_script(js):
        if 'jsApiReporter.finished' in js:
            return True
        if 'jsApiReporter.specResults' in js:
            return results
        if 'jsApiReporter.suiteResults' in js:
            return suite_results
        if 'jsApiReporter.runDetails' in js:
            return run_details_random
        return None

    firefox_driver.execute_script = execute_script

    CIRunner(jasmine_config=jasmine_config).run()
    stdout, _ = capsys.readouterr()
    assert 'Randomized with seed 12345 (jasmine-ci --seed 12345)' in stdout


def test_doesnt_displays_random_seed_if_not_random(results, suite_results, run_details, firefox_driver, capsys, test_server, jasmine_config, sysexit):
    def execute_script(js):
        if 'jsApiReporter.finished' in js:
            return True
        if 'jsApiReporter.specResults' in js:
            return results
        if 'jsApiReporter.suiteResults' in js:
            return suite_results
        if 'jsApiReporter.runDetails' in js:
            return run_details
        return None

    firefox_driver.execute_script = execute_script

    CIRunner(jasmine_config=jasmine_config).run()
    stdout, _ = capsys.readouterr()
    assert 'Randomized with seed' not in stdout




# coding=utf-8
import sys
import subprocess
from subprocess import Popen, PIPE
from time import sleep
import requests
from pytest import fail

def get_with_retries(url):
    n = 0
    while True:
        try:
            return requests.get(url)
        except requests.ConnectionError:
            if n < 20:
                n += 1
                sleep(0.1)
            else:
                raise

def test_standalone_serves_html():
    process = Popen([sys.executable, '-c', 'from jasmine.entry_points import begin; begin()', 'server', '--config', 'test/fixture_files/jasmine.yml'])
    try:
        req = get_with_retries('http://localhost:8888/')

        assert req.status_code == 200
        assert '/__jasmine__/jasmine.js' in req.text
        assert '/__src__/main.js' in req.text
        assert '/__src__/main.css' in req.text
        assert '/__spec__/someSpec.js' in req.text
    finally:
        process.terminate()

def test_standalone_supports_query_strings():
    process = Popen([sys.executable, '-c', 'from jasmine.entry_points import begin; begin()', 'server', '--config', 'test/fixture_files/jasmine.yml'])
    try:
        req = get_with_retries('http://localhost:8888/?random=true')

        assert req.status_code == 200
    finally:
        process.terminate()

def test_standalone_serves_jasmine_files():
    process = Popen([sys.executable, '-c', 'from jasmine.entry_points import begin; begin()', 'server', '--config', 'test/fixture_files/jasmine.yml'])
    try:
        req = get_with_retries('http://localhost:8888/__jasmine__/jasmine.css')

        assert 'text/css' in req.headers['content-type']
        assert req.status_code == 200
    finally:
        process.terminate()

def test_standalone_serves_jasmine_css_files():
    process = Popen([sys.executable, '-c', 'from jasmine.entry_points import begin; begin()', 'server', '--config', 'test/fixture_files/jasmine.yml'])
    try:
        req = get_with_retries('http://localhost:8888/__jasmine__/jasmine.js')

        assert 'application/javascript' in req.headers['content-type']
        assert req.status_code == 200
    finally:
        process.terminate()

def test_standalone_serves_jasmine_favicon():
    process = Popen([sys.executable, '-c', 'from jasmine.entry_points import begin; begin()', 'server', '--config', 'test/fixture_files/jasmine.yml'])
    try:
        req = get_with_retries('http://localhost:8888/__jasmine__/jasmine_favicon.png')

        assert 'image/png' in req.headers['content-type']
        assert req.status_code == 200
    finally:
        process.terminate()

def test_standalone_serves_js():
    process = Popen([sys.executable, '-c', 'from jasmine.entry_points import begin; begin()', 'server', '--config', 'test/fixture_files/jasmine.yml'])
    try:
        req = get_with_retries('http://localhost:8888/__src__/main.js')

        assert 'application/javascript' in req.headers['content-type']
        assert req.status_code == 200
        assert "thingUnderTest" in req.text
    finally:
        process.terminate()

def test_standalone_serves_css():
    process = Popen([sys.executable, '-c', 'from jasmine.entry_points import begin; begin()', 'server', '--config', 'test/fixture_files/jasmine.yml'])
    try:
        req = get_with_retries('http://localhost:8888/__src__/main.css')

        assert 'text/css' in req.headers['content-type']
        assert req.status_code == 200
        assert "CSS" in req.text
    finally:
        process.terminate()

def test_standalone_serves_weird_encodings():
    process = Popen([sys.executable, '-c', 'from jasmine.entry_points import begin; begin()', 'server', '--config', 'test/fixture_files/jasmine.yml'])
    try:
        req = get_with_retries('http://localhost:8888/__src__/weird_encoding.js')

        req.encoding = 'iso8859-1'
        assert req.status_code == 200

        assert u"weirdEncoding = 'ã'" in req.text
    finally:
        process.terminate()

def test_ci():
    if subprocess.call('which geckodriver > /dev/null', shell=True) != 0:
        fail('Geckodriver was not found in $PATH')

    process = Popen([sys.executable, '-c', 'from jasmine.entry_points import begin; begin()', 'ci', '--config', 'test/fixture_files/jasmine.yml'], stdout=PIPE)
    output = process.communicate()[0]
    process.wait()

    assert '2 specs, 0 failed' in str(output)
    assert process.returncode == 0

def test_help_fallback():
    process = Popen([sys.executable, '-c', 'from jasmine.entry_points import begin; begin()'], stdout=PIPE)
    output = process.communicate()[0]
    process.wait()

    assert 'usage:' in str(output)
    assert 'Jasmine command line' in str(output)
    assert process.returncode != 0



# -*- coding: utf-8 -*-
from __future__ import print_function
import os
import argparse
import socket
import sys
from jasmine.config import Config

from jasmine.standalone import JasmineApp
from jasmine.ci import CIRunner


def begin():
    cmd = Command(JasmineApp, CIRunner)
    cmd.run(sys.argv[1:])


def mkdir_p(path):
    import os
    import errno

    try:
        os.makedirs(path)
    except OSError as exc:  # Python >2.5
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise


class Command(object):
    def __init__(self, app, ci_runner):
        self.app = app
        self.ci_runner = ci_runner
        self.parser = argparse.ArgumentParser(description='Jasmine command line')
        subcommands = self.parser.add_subparsers(help='commands')
        server = subcommands.add_parser('server', help='Jasmine server',
                                        description='run a server hosting your Jasmine specs')
        server.add_argument('-p', '--port', type=int, default=8888,
                            help='The port of the Jasmine html runner')
        server.add_argument('-o', '--host', type=str, default='127.0.0.1',
                            help='The host of the Jasmine html runner')
        server.add_argument('-c', '--config', type=str,
                            help='Custom path to jasmine.yml')
        server.set_defaults(func=self.server)

        ci = subcommands.add_parser('ci', help='Jasmine CI', description='execute your specs in a browser')
        ci.add_argument('-b', '--browser', type=str,
                        help='The selenium driver to utilize')
        ci.add_argument('-l', '--logs', action='store_true',
                        help='Displays browser logs')
        ci.add_argument('-s', '--seed', type=str,
                        help='Seed for random spec order')
        ci.add_argument('-c', '--config', type=str,
                        help='Custom path to jasmine.yml')
        ci.set_defaults(func=self.ci)

        init = subcommands.add_parser('init', help='initialize Jasmine', description='')
        init.set_defaults(func=self.init)

    def run(self, argv):
        args = self.parser.parse_args(argv)
        if 'func' in args and callable(args.func):
            args.func(args)
        else:
            self.parser.print_help()
            self.parser.exit(1)

    def server(self, args):
        if self._check_for_config(args.config):
            jasmine_config = self._load_config(args.config)
            try:
                jasmine_app = self.app(jasmine_config=jasmine_config)
                jasmine_app.run(host=args.host, port=args.port, blocking=True)
            except socket.error:
                sys.stdout.write('Socket unavailable')

    def ci(self, args):
        if self._check_for_config(args.config):
            jasmine_config = self._load_config(args.config)
            jasmine_app = self.app(jasmine_config=jasmine_config)
            self.ci_runner(jasmine_config=jasmine_config).run(
                browser=args.browser,
                show_logs=args.logs,
                seed=args.seed,
                app=jasmine_app,
            )

    def _config_paths(self, custom_config_path):
        project_path = os.path.realpath(os.path.dirname(__name__))
        jasmine_conf = "spec/javascripts/support/jasmine.yml"
        if 'JASMINE_CONFIG_PATH' in os.environ:
            jasmine_conf = os.environ['JASMINE_CONFIG_PATH']
        if custom_config_path is not None:
            jasmine_conf = custom_config_path

        config_file = os.path.join(
            project_path,
            jasmine_conf
        )
        return config_file, project_path

    def _check_for_config(self, custom_config_path):
        config_file, _ = self._config_paths(custom_config_path)

        config_exists = os.path.exists(config_file)
        if not config_exists:
            print("Could not find your config file at {0}".format(config_file))
        return config_exists

    def _load_config(self, custom_config_path):
        config_file, project_path = self._config_paths(custom_config_path)
        return Config(config_file, project_path=project_path)

    def query(self, question):
        valid = {"yes": True, "y": True, "ye": True,
                 "no": False, "n": False}
        prompt = " [Y/n] "

        while True:
            sys.stdout.write(question + prompt)

            try:
                choice = raw_input().lower()
            except NameError:
                choice = input().lower()

            if choice == '':
                return True
            elif choice in valid:
                return valid[choice]

    def init(self, args):
        from jasmine.console_formatter import ConsoleFormatter

        spec_dir = os.path.join(os.getcwd(), 'spec/javascripts/')

        print(ConsoleFormatter.JASMINE_HEADER)

        print('Spec directory')

        msg = "About to create {0}... is this okay?".format(spec_dir)
        if self.query(msg):
            print('making spec/javascripts')
            mkdir_p(spec_dir)

        yaml_dir = os.path.join(spec_dir, 'support')
        yaml_file_path = os.path.join(yaml_dir, 'jasmine.yml')

        print(("*" * 80) + '\n\nConfig yaml')

        if os.path.exists(yaml_file_path):
            print('found existing {0}, not overwriting'.format(yaml_file_path))
        else:
            msg = "About to create {0}... is this okay?".format(yaml_file_path)
            if self.query(msg):
                print('making {0}'.format(yaml_dir))

                mkdir_p(yaml_dir)

                print('making {0}'.format(yaml_file_path))
                try:
                    with open(yaml_file_path, 'w') as f:
                        f.write(self.YAML_TEMPLATE)
                        f.flush()
                except IOError:
                    pass

    YAML_TEMPLATE = """
    # src_files
    #
    # Return an array of filepaths relative to src_dir to include before jasmine specs.
    # Default: []
    #
    # EXAMPLE:
    #
    # src_files:
    #   - lib/source1.js
    #   - lib/source2.js
    #   - dist/**/*.js
    #
    src_files:

    # stylesheets
    #
    # Return an array of stylesheet filepaths relative to src_dir to include before jasmine specs.
    # Default: []
    #
    # EXAMPLE:
    #
    # stylesheets:
    #   - css/style.css
    #   - stylesheets/*.css
    #
    stylesheets:

    # helpers
    #
    # Return an array of filepaths relative to spec_dir to include before jasmine specs.
    # Default: ["helpers/**/*.js"]
    #
    # EXAMPLE:
    #
    # helpers:
    #   - helpers/**/*.js
    #
    helpers:
      - "helpers/**/*.js"

    # spec_files
    #
    # Return an array of filepaths relative to spec_dir to include.
    # Default: ["**/*[sS]pec.js"]
    #
    # EXAMPLE:
    #
    # spec_files:
    #   - **/*[sS]pec.js
    #
    spec_files:
      - "**/*[Ss]pec.js"

    # src_dir
    #
    # Source directory path. Your src_files must be returned relative to this path. Will use root if left blank.
    # Default: project root
    #
    # EXAMPLE:
    #
    # src_dir: public
    #
    src_dir:

    # spec_dir
    #
    # Spec directory path. Your spec_files must be returned relative to this path.
    # Default: spec/javascripts
    #
    # EXAMPLE:
    #
    # spec_dir: spec/javascripts
    #
    spec_dir: spec/javascripts

    # stop_spec_on_expectation_failure
    #
    # Stop executing each spec on the first expectation failure.
    # Default: false
    #
    # EXAMPLE:
    #
    # stop_spec_on_expectation_failure: true
    #
    stop_spec_on_expectation_failure:

    # stop_on_spec_failure
    #
    # Stop executing Jasmine after the first spec fails
    # Default: false
    #
    # EXAMPLE:
    #
    # stop_on_spec_failure: true
    #
    stop_on_spec_failure:

    # random
    #
    # Run specs in semi-random order.
    # Default: true
    #
    # EXAMPLE:
    #
    # random: false
    #
    random:
    """



import pytest

import urllib

from jasmine.url_builder import JasmineUrlBuilder
from test.helpers.fake_config import FakeConfig


@pytest.fixture
def jasmine_config():
    return FakeConfig(
        src_dir='src',
        spec_dir='spec',
    )


def test_defaults(jasmine_config):
    url = JasmineUrlBuilder(jasmine_config=jasmine_config).build_url(port=80)

    assert url == 'http://localhost:80?random=true'


def test_stop_spec_on_expectation_failure(jasmine_config):
    jasmine_config._stop_spec_on_expectation_failure = True
    url = JasmineUrlBuilder(jasmine_config=jasmine_config).build_url(port=80)
    uri_tuple = urllib.parse.urlparse(url)
    assert uri_tuple[0] == 'http'
    assert uri_tuple[1] == 'localhost:80'
    assert urllib.parse.parse_qs(uri_tuple[4]) == {"random": ['true'], "throwFailures": ['true']}


def test_stop_on_spec_failure(jasmine_config):
    jasmine_config._stop_on_spec_failure = True
    url = JasmineUrlBuilder(jasmine_config=jasmine_config).build_url(port=80)
    uri_tuple = urllib.parse.urlparse(url)
    assert uri_tuple[0] == 'http'
    assert uri_tuple[1] == 'localhost:80'
    assert urllib.parse.parse_qs(uri_tuple[4]) == {"random": ['true'], "failFast": ['true']}


def test_random(jasmine_config):
    jasmine_config._random = False
    false_url = JasmineUrlBuilder(jasmine_config=jasmine_config).build_url(port=80)
    assert false_url == 'http://localhost:80?random=false'


def test_seed(jasmine_config):
    url = JasmineUrlBuilder(jasmine_config=jasmine_config).build_url(port=80, seed=1234)
    uri_tuple = urllib.parse.urlparse(url)
    params = urllib.parse.parse_qs(uri_tuple[4])
    assert params['seed'] == ['1234']


def test_multiple_query_parameters(jasmine_config):
    jasmine_config._random = True
    jasmine_config._stop_spec_on_expectation_failure = True
    jasmine_config._stop_on_spec_failure = True

    url = JasmineUrlBuilder(jasmine_config=jasmine_config).build_url(port=80, seed=1234)

    uri_tuple = urllib.parse.urlparse(url)
    assert uri_tuple[0] == 'http'
    assert uri_tuple[1] == 'localhost:80'
    assert urllib.parse.parse_qs(uri_tuple[4]) == {"random": ['true'], "throwFailures": ['true'], "failFast": ['true'], "seed": ['1234']}




# coding=utf-8

# Python License, Version 2 (Python-2.0)
#
# PYTHON SOFTWARE FOUNDATION LICENSE VERSION 2
#
# Licensed to Éric Araujo <merwok@netwok.org>
#
# Lovingly copied from
# http://hg.python.org/distutils2/file/65e23cccd0db/distutils2/util.py

try:
    from glob import iglob as std_iglob
except ImportError:
    from glob import glob as std_iglob
import re
import os

RICH_GLOB = re.compile(r'\{([^}]*)\}')
_CHECK_RECURSIVE_GLOB = re.compile(r'[^/\\,{]\*\*|\*\*[^/\\,}]')
_CHECK_MISMATCH_SET = re.compile(r'^[^{]*\}|\{[^}]*$')


def iglob(path_glob):
    """Extended globbing function that supports ** and {opt1,opt2,opt3}."""
    if _CHECK_RECURSIVE_GLOB.search(path_glob):
        msg = """invalid glob %r: recursive glob "**" must be used alone"""
        raise ValueError(msg % path_glob)
    if _CHECK_MISMATCH_SET.search(path_glob):
        msg = """invalid glob %r: mismatching set marker '{' or '}'"""
        raise ValueError(msg % path_glob)
    return _iglob(path_glob)


def _iglob(path_glob):
    rich_path_glob = RICH_GLOB.split(path_glob, 1)
    if len(rich_path_glob) > 1:
        assert len(rich_path_glob) == 3, rich_path_glob
        prefix, set, suffix = rich_path_glob
        for item in set.split(','):
            for path in _iglob(''.join((prefix, item, suffix))):
                yield path
    else:
        if '**' not in path_glob:
            for item in std_iglob(path_glob):
                yield item
        else:
            prefix, radical = path_glob.split('**', 1)
            if prefix == '':
                prefix = '.'
            if radical == '':
                radical = '*'
            else:
                # we support both
                radical = radical.lstrip('/')
                radical = radical.lstrip('\\')
            for path, dir, files in os.walk(prefix):
                path = os.path.normpath(path)
                for file in _iglob(os.path.join(path, radical)):
                    yield file



import itertools
import os
import socket
import sys
import threading

from selenium.common.exceptions import WebDriverException
from selenium.webdriver.support.wait import WebDriverWait

from jasmine.console_formatter import ConsoleFormatter
from jasmine.js_api_parser import Parser
from jasmine.url_builder import JasmineUrlBuilder


class TestServerThread(threading.Thread):

    def __init__(self, port, app=None, *args, **kwargs):
        super(TestServerThread, self).__init__(*args, **kwargs)
        self.port = port
        self.app = app

    def run(self):
        self.app.run('127.0.0.1', self.port, blocking=False)

    def join(self, timeout=None):
        self.app.stop()

    def _possible_ports(self):
        return itertools.chain(range(80, 81, 1), range(8889, 10000))


class CIRunner(object):

    def __init__(self, jasmine_config=None):
        self.jasmine_config = jasmine_config

    def run(self, browser=None, show_logs=False, app=None, seed=None):
        try:
            port = self._find_unused_port()
            self.browser = self._get_browser(browser)
            self.test_server = self._start_test_server(app, port)

            url_builder = JasmineUrlBuilder(jasmine_config=self.jasmine_config)
            jasmine_url = url_builder.build_url(port, seed)
            self.browser.get(jasmine_url)

            WebDriverWait(self.browser, 100).until(
                lambda driver:
                driver.execute_script("return jsApiReporter.finished;")
            )

            parser = Parser()
            spec_results = self._get_spec_results(parser)
            top_suite_results = self._get_top_suite_results(parser)
            suite_results = self._get_suite_results(parser) + top_suite_results
            show_logs = self._get_browser_logs(show_logs=show_logs)
            actual_seed = self._get_seed()

            formatter = ConsoleFormatter(
                spec_results=spec_results,
                suite_results=suite_results,
                browser_logs=show_logs,
                seed=actual_seed
            )
            str_output = formatter.format()
            if sys.version_info[0] < 3:
                sys.stdout.write(str_output.encode('UTF8'))
            else:
                sys.stdout.write(str_output)

            overall_status = self._get_overall_status()

            if overall_status == 'incomplete':
                sys.stdout.write('Incomplete: %s\n' % self._get_incomplete_reason())

            if overall_status != 'passed':
                sys.exit(1)
        finally:
            if hasattr(self, 'browser'):
                self.browser.close()
            if hasattr(self, 'test_server'):
                self.test_server.join()

    def _find_unused_port(self):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.bind(('0.0.0.0', 0))
        addr, port = s.getsockname()
        s.close()
        return port

    def _start_test_server(self, app, port):
        test_server = TestServerThread(port, app=app)
        test_server.start()
        return test_server

    def _get_browser(self, browser):
        driver = browser if browser \
            else os.environ.get('JASMINE_BROWSER', 'firefox')
        try:
            webdriver = __import__(
                "selenium.webdriver.{0}.webdriver".format(driver),
                globals(), locals(), ['object'], 0
            )

            return webdriver.WebDriver()
        except ImportError:
            print("Browser {0} not found".format(driver))

    def _get_spec_results(self, parser):
        spec_results = []
        index = 0
        batch_size = 50
        while True:
            results = self.browser.execute_script(
                "return jsApiReporter.specResults({0}, {1})".format(
                    index,
                    batch_size
                )
            )
            spec_results.extend(results)
            index += len(results)

            if not len(results) == batch_size:
                break

        return parser.parse(spec_results)

    def _get_suite_results(self, parser):
        suite_results = []
        index = 0
        batch_size = 50
        while True:
            results = self.browser.execute_script(
                "return jsApiReporter.suiteResults({0}, {1})".format(
                    index,
                    batch_size
                )
            )

            suite_results.extend(results)
            index += len(results)

            if not len(results) == batch_size:
                break

        return parser.parse(suite_results)

    def _get_overall_status(self):
        return self.browser.execute_script('return jsApiReporter.runDetails').get('overallStatus')

    def _get_incomplete_reason(self):
        return self.browser.execute_script('return jsApiReporter.runDetails').get('incompleteReason')

    def _get_top_suite_results(self, parser):
        failed_expectations = self.browser.execute_script("return jsApiReporter.runDetails").get('failedExpectations')

        top_suite_result = {
            "failedExpectations": failed_expectations,
            "status": "failed" if len(failed_expectations) else "passed"
        }

        return parser.parse([top_suite_result])

    def _get_seed(self):
        order = self._get_order()
        is_random = order.get('random')
        seed = order.get('seed') if is_random else None
        return seed

    def _get_order(self):
        return self.browser.execute_script("return jsApiReporter.runDetails").get('order')

    def _get_browser_logs(self, show_logs):
        log = []
        if show_logs:
            try:
                log = self.browser.get_log('browser')
            except WebDriverException:
                pass
        return log



from setuptools import setup, find_packages

setup(
    name="jasmine",
    version="3.6.0",
    url="http://jasmine.github.io",
    author="Pivotal Labs",
    author_email="jasmine-js@googlegroups.com",
    description=('Jasmine is a Behavior Driven Development'
                 ' testing framework for JavaScript.'
                 ' It does not rely on browsers, DOM,'
                 ' or any JavaScript framework.'
                 ' Thus it\'s suited for websites, '
                 'Node.js (http://nodejs.org) projects,'
                 ' or anywhere that JavaScript can run.'),
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Quality Assurance',
        'Topic :: Software Development :: Testing',
    ],

    packages=find_packages(),
    package_data={'jasmine.templates': ['*.html']},
    include_package_data=True,

    entry_points={
        'console_scripts': [
            'jasmine = jasmine.entry_points:begin'
        ]
    },

    install_requires=[
        'PyYAML>=4.2b1',
        'Jinja2>=2.0, <3.0',
        'jasmine-core>=3.6.0, <4.0',
        'CherryPy>=11',
        'selenium>=3.0',
    ],
)



from jasmine.result_list import ResultList


class Parser(object):
    RESULT_FIELDS = {
        'status': 'status',
        'fullName': 'full_name',
        'failedExpectations': 'failed_expectations',
        'deprecationWarnings': 'deprecation_warnings',
        'id': 'runnable_id',
        'description': 'description',
        'pendingReason': 'pending_reason'
    }

    def parse(self, items):
        result_list = ResultList()
        for item in self._filter_fields(items):
            result_list.add_result(item)
        return result_list

    def _filter_fields(self, raw_items):
        filtered_items = []
        for item in raw_items:
            filtered_items.append(dict((
                (self._to_snake_case(k), v)
                for k, v in item.items()
                if k in self.RESULT_FIELDS.keys()
            )))
        return filtered_items

    def _to_snake_case(self, key):
        return self.RESULT_FIELDS[key]
